==> roccat-tools-1.0.0.tar.bz2: roccat-tools-1.0.0/savu/libsavu/savu_general.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/savu.h"
#include "roccat_helper.h"
#include <string.h>
#include <stdlib.h>

static gchar const * const general_name = "general";

guint16 savu_general_calc_checksum(SavuGeneral const *general) {
	return ROCCAT_BYTESUM_PARTIALLY(general, SavuGeneral, command, checksum);
}

static void savu_general_finalize(SavuGeneral *general, guint profile_index) {
	general->command = SAVU_COMMAND_GENERAL;
	general->size = sizeof(SavuGeneral);
	general->profile_index = profile_index;
	savu_general_set_checksum(general, savu_general_calc_checksum(general));
}

SavuGeneral *savu_general_read(RoccatDevice *savu, guint profile_index, GError **error) {
	SavuGeneral *general;

	g_assert(profile_index < SAVU_PROFILE_NUM);

	roccat_device_lock(savu);

	if (!savu_select(savu, profile_index, SAVU_CONTROL_REQUEST_GENERAL, error)) {
		roccat_device_unlock(savu);
		return NULL;
	}

	general = (SavuGeneral *)roccat_device_read(savu, general_name, sizeof(SavuGeneral), error);

	roccat_device_unlock(savu);

	return general;
}

gboolean savu_general_write(RoccatDevice *savu, guint profile_index, SavuGeneral *general, GError **error) {
	g_assert(profile_index < SAVU_PROFILE_NUM);
	savu_general_finalize(general, profile_index);
	return roccat_device_write(savu, general_name, (gchar const *)general, sizeof(SavuGeneral), error);
}

static guint8 savu_rmp_get_cpi_mask(SavuRmp *rmp) {
	guint8 result = 0;
	guint i;

	for (i = 0; i < SAVU_GENERAL_CPI_LEVELS_NUM; ++i)
		roccat_set_bit(&result, i, savu_rmp_get_cpi(rmp, i));

	return result;
}

static void savu_rmp_update_with_cpi_mask(SavuRmp *rmp, guint8 cpi_mask) {
	guint i;

	for (i = 0; i < SAVU_GENERAL_CPI_LEVELS_NUM; ++i)
		savu_rmp_set_cpi(rmp, i, roccat_get_bit(cpi_mask, i));
}

static guint8 savu_rmp_get_lighting_mask(SavuRmp *rmp) {
	guint8 result = 0;

	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_LIGHT, savu_rmp_get_light_switch(rmp));
	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_BREATHING, savu_rmp_get_breath_on(rmp));
	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_COLOR_FLOW, savu_rmp_get_color_flow(rmp));

	return result;
}

static void savu_rmp_update_with_lighting_mask(SavuRmp *rmp, guint8 lighting_mask) {
	savu_rmp_set_light_switch(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_LIGHT));
	savu_rmp_set_breath_on(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_BREATHING));
	savu_rmp_set_color_flow(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_COLOR_FLOW));
}

SavuGeneral *savu_rmp_to_general(SavuRmp *rmp) {
	SavuGeneral *general;

	general = (SavuGeneral *)g_malloc0(sizeof(SavuGeneral));

	general->sensitivity_sync = savu_rmp_get_xy_synchronous(rmp);
	general->sensitivity_x = savu_rmp_get_sensitivity_x(rmp);
	general->sensitivity_y = savu_rmp_get_sensitivity_y(rmp);
	general->polling_rate = savu_rmp_get_polling_rate(rmp);
	general->color_index = savu_rmp_get_color_database(rmp);
	general->red = savu_rmp_get_color_red(rmp);
	general->green = savu_rmp_get_color_green(rmp);
	general->blue = savu_rmp_get_color_blue(rmp);
	general->cpi = savu_rmp_get_cpi_step(rmp);
	general->cpi_mask = savu_rmp_get_cpi_mask(rmp);
	general->lighting_mask = savu_rmp_get_lighting_mask(rmp);

	return general;
}

void savu_rmp_update_with_general(SavuRmp *rmp, SavuGeneral *general) {
	savu_rmp_set_xy_synchronous(rmp, general->sensitivity_sync);
	savu_rmp_set_sensitivity_x(rmp, general->sensitivity_x);
	savu_rmp_set_sensitivity_y(rmp, general->sensitivity_y);
	savu_rmp_set_polling_rate(rmp, general->polling_rate);
	savu_rmp_set_color_database(rmp, general->color_index);
	savu_rmp_set_color_red(rmp, general->red);
	savu_rmp_set_color_green(rmp, general->green);
	savu_rmp_set_color_blue(rmp, general->blue);
	savu_rmp_set_cpi_step(rmp, general->cpi);
	savu_rmp_update_with_cpi_mask(rmp, general->cpi_mask);
	savu_rmp_update_with_lighting_mask(rmp, general->lighting_mask);
}

==> roccat-tools-1.6.0.tar.bz2: roccat-tools-1.6.0/savu/libsavu/savu_general.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "savu.h"
#include "roccat_helper.h"
#include "roccat_device_sysfs.h"
#include <string.h>
#include <stdlib.h>

static gchar const * const general_name = "general";

guint16 savu_general_calc_checksum(SavuGeneral const *general) {
	return ROCCAT_BYTESUM_PARTIALLY(general, SavuGeneral, command, checksum);
}

static void savu_general_finalize(SavuGeneral *general, guint profile_index) {
	general->command = SAVU_COMMAND_GENERAL;
	general->size = sizeof(SavuGeneral);
	general->profile_index = profile_index;
	savu_general_set_checksum(general, savu_general_calc_checksum(general));
}

SavuGeneral *savu_general_read(RoccatDevice *savu, guint profile_index, GError **error) {
	SavuGeneral *general;

	g_assert(profile_index < SAVU_PROFILE_NUM);

	roccat_device_lock(savu);

	if (!savu_select(savu, profile_index, SAVU_CONTROL_REQUEST_GENERAL, error)) {
		roccat_device_unlock(savu);
		return NULL;
	}

	general = (SavuGeneral *)roccat_device_sysfs_read(savu, general_name, sizeof(SavuGeneral), error);

	roccat_device_unlock(savu);

	return general;
}

gboolean savu_general_write(RoccatDevice *savu, guint profile_index, SavuGeneral *general, GError **error) {
	g_assert(profile_index < SAVU_PROFILE_NUM);
	savu_general_finalize(general, profile_index);
	return roccat_device_sysfs_write(savu, general_name, (gchar const *)general, sizeof(SavuGeneral), error);
}

static guint8 savu_rmp_get_cpi_mask(SavuRmp *rmp) {
	guint8 result = 0;
	guint i;

	for (i = 0; i < SAVU_GENERAL_CPI_LEVELS_NUM; ++i)
		roccat_set_bit(&result, i, savu_rmp_get_cpi(rmp, i));

	return result;
}

static void savu_rmp_update_with_cpi_mask(SavuRmp *rmp, guint8 cpi_mask) {
	guint i;

	for (i = 0; i < SAVU_GENERAL_CPI_LEVELS_NUM; ++i)
		savu_rmp_set_cpi(rmp, i, roccat_get_bit(cpi_mask, i));
}

static guint8 savu_rmp_get_lighting_mask(SavuRmp *rmp) {
	guint8 result = 0;

	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_LIGHT, savu_rmp_get_light_switch(rmp));
	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_BREATHING, savu_rmp_get_breath_on(rmp));
	roccat_set_bit(&result, SAVU_GENERAL_LIGHTING_MASK_BIT_COLOR_FLOW, savu_rmp_get_color_flow(rmp));

	return result;
}

static void savu_rmp_update_with_lighting_mask(SavuRmp *rmp, guint8 lighting_mask) {
	savu_rmp_set_light_switch(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_LIGHT));
	savu_rmp_set_breath_on(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_BREATHING));
	savu_rmp_set_color_flow(rmp, roccat_get_bit(lighting_mask, SAVU_GENERAL_LIGHTING_MASK_BIT_COLOR_FLOW));
}

SavuGeneral *savu_rmp_to_general(SavuRmp *rmp) {
	SavuGeneral *general;

	general = (SavuGeneral *)g_malloc0(sizeof(SavuGeneral));

	general->sensitivity_sync = savu_rmp_get_xy_synchronous(rmp);
	general->sensitivity_x = savu_rmp_get_sensitivity_x(rmp);
	general->sensitivity_y = savu_rmp_get_sensitivity_y(rmp);
	general->polling_rate = savu_rmp_get_polling_rate(rmp);
	general->color_index = savu_rmp_get_color_database(rmp);
	general->red = savu_rmp_get_color_red(rmp);
	general->green = savu_rmp_get_color_green(rmp);
	general->blue = savu_rmp_get_color_blue(rmp);
	general->cpi = savu_rmp_get_cpi_step(rmp);
	general->cpi_mask = savu_rmp_get_cpi_mask(rmp);
	general->lighting_mask = savu_rmp_get_lighting_mask(rmp);

	return general;
}

void savu_rmp_update_with_general(SavuRmp *rmp, SavuGeneral *general) {
	savu_rmp_set_xy_synchronous(rmp, general->sensitivity_sync);
	savu_rmp_set_sensitivity_x(rmp, general->sensitivity_x);
	savu_rmp_set_sensitivity_y(rmp, general->sensitivity_y);
	savu_rmp_set_polling_rate(rmp, general->polling_rate);
	savu_rmp_set_color_database(rmp, general->color_index);
	savu_rmp_set_color_red(rmp, general->red);
	savu_rmp_set_color_green(rmp, general->green);
	savu_rmp_set_color_blue(rmp, general->blue);
	savu_rmp_set_cpi_step(rmp, general->cpi);
	savu_rmp_update_with_cpi_mask(rmp, general->cpi_mask);
	savu_rmp_update_with_lighting_mask(rmp, general->lighting_mask);
}
